import os
import shutil

import click

from altius_l1pp_inflight.configuration.altius_config import user_defined_config_dir


@click.command()
@click.option('-d', '--matlab_directory', type=str, help='Full path to copy the ALTIUS L1PP Inflight MatLab files')
@click.option('--config', type=str, is_flag=True, help='Copy ')
def cli(matlab_directory, config):
    copy_matlab_files(matlab_directory)
    if config:
        copy_example_config_files()


def copy_example_config_files():
    srcdir = os.path.abspath(os.path.dirname(__file__))
    try:
        test_dest = os.path.join(user_defined_config_dir, 'test_globalkey.yaml')
        glob_dest = os.path.join(user_defined_config_dir, 'globalkey.yaml')
        if os.path.exists(test_dest) or os.path.exists(glob_dest):
            click.echo(click.style('Okay to move existing configuration files? (y/n)', fg='yellow'))
            yesno = input()
            if yesno == 'y':
                d = user_defined_config_dir
                if d[-1] == '/' or d[-1] == '\\':
                    d = d[:-1]
                old_files_new_dest = d + '_old'
                shutil.move(d, old_files_new_dest)
                os.makedirs(user_defined_config_dir)  # fresh start
                click.secho('Moved old config files from {} to {}, consider deleting them'.format(d, old_files_new_dest),
                            fg='yellow')
            else:
                click.secho('Skipped copying config files. Consider moving them.  Exiting.', fg='red')
                return
        shutil.copy2(os.path.join(srcdir, 'test_globalkey.yaml'), user_defined_config_dir)
        shutil.copy2(os.path.join(srcdir, 'globalkey.yaml'), user_defined_config_dir)
    except Exception as e:
        click.secho('Error: {}'.format(e), fg='red')
        click.secho('Possible fix: rename or move old test_globalkey.yaml and globalkey.yaml files', fg='yellow')
    finally:
        click.secho('Finished copying test_globalkey.yaml and globalkey.yaml '
                    'to local config folder {}'.format(user_defined_config_dir), fg='green')


def copy_matlab_files(matlab_directory):
    src = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'MATLAB'))

    if os.path.exists(matlab_directory):
        click.echo(click.style('Okay to move existing MatLab calibration files? (y/n)', fg='yellow'))
        yesno = input()
        if yesno == 'y':
            d = matlab_directory
            if d[-1] == '/' or d[-1] == '\\':
                d = d[:-1]

            old_files_new_dest = d + '_old'
            shutil.move(matlab_directory, old_files_new_dest)
            click.echo(click.style('Old MatLab files moved to {}, considering removing'.format(old_files_new_dest),
                                   fg='yellow'))
            shutil.copytree(src, matlab_directory)
            click.echo(click.style('New MatLab files moved to {}'.format(matlab_directory), fg='green'))
        else:
            click.echo(click.style('Skipped copying MatLab files. Consider moving them.  Exiting.', fg='red'))
    else:
        shutil.copytree(src, matlab_directory)
        click.echo(click.style('New MatLab files created at {}'.format(matlab_directory), fg='green'))


if __name__ == '__main__':
    copy_matlab_files('C:/tmp/')
