import numpy as np

# TODO can improve on the structure of this function
from altius_l1pp_inflight.scene.disk_geometry import SolarGeometry
from altius_l1pp_inflight.scene.limbdark_mu_pierceandslaughter1977 import LimbDarkening_PS1977


def compute_irradiance_center_pixel_scaling_factor(wavelengths: np.ndarray,
                                                   radius_pixels: float,
                                                   center_radius_pixels: float,
                                                   solar_distance_km: float,
                                                   horizontal_center_pixels: float,
                                                   vertical_center_pixels: float,
                                                   horizontal_image_width: int,
                                                   vertical_image_width: int,
                                                   horizontal_boresight_pixel: float,
                                                   vertical_boresight_pixel: float):
    """
    Passes in the dimensions of the detector to mimic the measurement - we want the exact
    same set of pixels in the center to be used as the measurement
    """
    sample_hr_wl = np.arange(wavelengths[0] - 10, wavelengths[-1] + 10, step=2)
    irradiance_scaling_factor = np.zeros_like(sample_hr_wl)

    solar_geometry = SolarGeometry(horizontal_image_width, vertical_image_width, vertical_boresight_pixel,
                                   horizontal_boresight_pixel)
    solar_geometry.set_disk(radius_pixels, horizontal_center_pixels, vertical_center_pixels, solar_distance_km)

    mu = solar_geometry._mu()
    h = np.arange(0, horizontal_image_width) + 0.5 - horizontal_center_pixels
    v = np.arange(0, vertical_image_width) + 0.5 - vertical_center_pixels
    hh, vv = np.meshgrid(h, v)
    pixel_radii = np.sqrt(vv ** 2 + hh ** 2)
    mask = pixel_radii <= center_radius_pixels
    # TODO can add this to geometry class

    h_center = np.arange(0, horizontal_image_width) + 0.5 - horizontal_boresight_pixel
    v_center = np.arange(0, vertical_image_width) + 0.5 - vertical_boresight_pixel
    hh_c, vv_c = np.meshgrid(h_center, v_center)

    # Correction for off-axis viewing
    delta_omega_view, _, _ = solar_geometry.delta_omega_view()
    delta_omega_center, _, _ = solar_geometry._delta_omega_center()

    for idx, wl in enumerate(sample_hr_wl):
        irradiance_scaling_factor[idx] = _compute_irradiance_center_pixel_scaling_factor_wl(wl, mu,
                                                                                            delta_omega_center,
                                                                                            delta_omega_view,
                                                                                            mask)
    irradiance_scaling_factor = np.interp(wavelengths, sample_hr_wl, irradiance_scaling_factor)
    return irradiance_scaling_factor


def _compute_irradiance_center_pixel_scaling_factor_wl(wavelength: float,
                                                       mu: np.ndarray,
                                                       delta_omega_center: np.ndarray,
                                                       delta_omega_view: np.ndarray,
                                                       mask: np.ndarray):
    limb_darkening_model = LimbDarkening_PS1977()
    mu_profile = np.linspace(0, 1, 50)

    ld_profile = limb_darkening_model.limb_darkening(wavelength, mu_profile)
    ld = np.interp(mu, mu_profile, ld_profile)
    ld *= delta_omega_center / delta_omega_view
    irradiance_scaling_factor = np.nansum(ld[mask]) / np.nansum(ld)

    return irradiance_scaling_factor
