import os

import matplotlib.pyplot as plt
import numpy as np
from sasktran.solarspectrum import SolarSpectrum

from altius_l1pp_inflight.configuration import altius_config
from altius_l1pp_inflight.scene.disk_geometry import D_1AU
from altius_l1pp_inflight.scene.generation.solar.hmi_files import HmiFileManager, HmiSunspotScenario, \
    SolarDiskRadianceResult
from altius_l1pp_inflight.scene.limbdark_mu_pierceandslaughter1977 import LimbDarkening_PS1977

hr_wavelen = np.linspace(300, 350, 5001)  # 0.01 nm res
analysis_wavelength = 325
fwhm = 3.0
sigma = fwhm / (2 * np.sqrt(2 * np.log(2)))
srf = np.exp((hr_wavelen - analysis_wavelength) ** 2 / (2 * sigma ** 2))  # TODO this will be passed in
srf /= np.sum(srf)


# Simple function for correcting the limb darkening of an HMI solar image and scaling it to the correct irradiance

def test_disk_radiance():
    sao = SolarSpectrum()
    continuum_folder = 'A:/USASK/ALTIUS/altius_l1pp/altius_l1pp/altius_prnu/solar/' \
                       'scene_generation/downloads/continuum/two_sunspots'
    fullname = os.path.join(continuum_folder, 'hmi.ic_720s.20160405_000000_TAI.1.continuum.fits')
    ok = os.path.exists(fullname)

    all_results = HmiFileManager(altius_config.AltiusL1ppInflightConfig()).load_scenario_files_as_images(HmiSunspotScenario.SEVERAL_SUNSPOTS)
    result: SolarDiskRadianceResult = all_results[0]
    radiance_disk_617_hmi = result.radiance()
    x_c, y_c = result.center_pixel()
    average_radius_pixels = result.average_solar_radius_in_pixels()
    actual_solar_distance = result.actual_solar_distance()
    mu = result.mu_solar_disk()
    delta_omega = result.delta_omega()

    # plt.figure()
    # plt.imshow(np.reshape(np.rad2deg(disk_radii_angles), (4096, 4096)))
    limb_darkening = LimbDarkening_PS1977()

    mu_profile = np.linspace(0, 1.0, 50)
    ld_617_profile = limb_darkening.limb_darkening(617.3, mu_profile)

    ld_617 = np.interp(mu, mu_profile, ld_617_profile)

    flattened_radiance = radiance_disk_617_hmi / ld_617

    plt.figure()
    plt.imshow(np.fliplr(radiance_disk_617_hmi))
    plt.colorbar()

    plt.figure()
    plt.imshow(np.fliplr(flattened_radiance))
    plt.colorbar()

    ld_335_profile = limb_darkening.limb_darkening(335, mu_profile)
    ld_335 = np.interp(mu, mu_profile, ld_335_profile)
    radiance_disk_335 = flattened_radiance * ld_335

    plt.figure()
    plt.imshow(np.fliplr(radiance_disk_335))
    plt.colorbar()

    plt.figure()
    plt.plot(radiance_disk_617_hmi[int(x_c), :])
    plt.plot(radiance_disk_335[int(x_c), :])
    plt.legend(['617.3 nm', '335 nm'])
    plt.title('Cross section solar radiance (DN)')

    plt.figure()
    plt.plot(flattened_radiance[int(x_c), :])
    plt.title('Flattened radiance X-section')

    # Compute the integral across the disk
    # TODO better integrations than just sums?
    integrated_disk_irradiance = np.sum(radiance_disk_335 * delta_omega)

    soa_irradiance = sao.irradiance(wavelengths=hr_wavelen, solardistance=actual_solar_distance / D_1AU)
    irrad = np.sum(soa_irradiance * srf)

    absolute_radiance_disk_335 = radiance_disk_335 * irrad / integrated_disk_irradiance

    print('Irradiance @ 335 nm: ', irrad, ' photons/s/nm/m2')

    plt.figure()
    plt.imshow(absolute_radiance_disk_335)
    plt.title('Absolute Radiance of Sun @ 335 nm')
    plt.colorbar()
    plt.show()


if __name__ == '__main__':
    test_disk_radiance()
