import os

import matplotlib.pyplot as plt
import numpy as np

from altius_l1pp_inflight.configuration import altius_config
from altius_l1pp_inflight.scene.altius_scene import SpsToUsaskSolarSceneInterface
from altius_l1pp_inflight.scene.generation.solar.hmi_files import HmiSunspotScenario


def get_altius_output_file(config_filename):
    config = altius_config.AltiusL1ppInflightConfig(config_filename)
    home_dir = config.get_processed_solar_prnu_file_directory()
    num_vert_scans = config.get_num_vertical_raster_scans()
    num_horz_scans = config.get_num_horizontal_raster_scans()
    raster_grid_str = str(num_horz_scans) + 'x' + str(num_vert_scans)
    data_dir = os.path.join(home_dir, 'altius_observation', raster_grid_str,
                            HmiSunspotScenario.NO_SUNSPOTS.to_string_directory())
    sps_to_usask_interface = SpsToUsaskSolarSceneInterface()
    return sps_to_usask_interface, data_dir


config_filename = 'globalkey_10x10_grid.yaml'
sps_to_usask_interface_10x10, data_dir_10x10 = get_altius_output_file(config_filename)
sps_to_usask_interface_10x10.open_read_dataset(os.path.join(data_dir_10x10, 'prnu_calibration_observations.nc'))
scene10x10 = sps_to_usask_interface_10x10.load_scene(40)

config_filename = 'globalkey_8x8_grid.yaml'
sps_to_usask_interface_8x8, data_dir_8x8 = get_altius_output_file(config_filename)
sps_to_usask_interface_8x8.open_read_dataset(os.path.join(data_dir_8x8, 'prnu_calibration_observations.nc'))
scene8x8 = sps_to_usask_interface_8x8.load_scene(20)

span = int(scene8x8.get_expected_altius_solar_radius() * 1.2)

rad8x8 = scene8x8.get_altius_image()
hc8 = int(np.round(scene8x8.get_expected_altius_solar_center_horz()))
vc8 = int(np.round(scene8x8.get_expected_altius_solar_center_vert()))

rad12x12 = scene10x10.get_altius_image()
hc12 = int(np.round(scene10x10.get_expected_altius_solar_center_horz()))
vc12 = int(np.round(scene10x10.get_expected_altius_solar_center_vert()))

plt.figure()
plt.imshow(rad8x8)
plt.colorbar()
plt.figure()
plt.imshow(rad12x12)
plt.colorbar()

plt.figure()
plt.plot(rad8x8[vc8, hc8 - span:hc8 + span] / np.nanmax(rad8x8), alpha=0.9, linewidth=0.5)
plt.plot(rad12x12[vc12, hc12 - span:hc12 + span] / np.nanmax(rad12x12), alpha=0.9, linewidth=0.5)
plt.show()
